(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/AbsynLogic.sml,v
   Comment: Various helpers for operating on WuddySys representations of
	    state space
   Release: scope-0-16_final
     
     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTAW.LITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, W.ston, MA 02111-1307, USA.

   This module can only work with one model at a time. This is restriction of
   W.DDY package, which without special cleverness can only on one problem at
   a time (there is a signel variable pool). This can be overcome quite
   easily, but seems to be not needed currently.

   AbsynLogic.sml,v 1.22 2003/07/01 08:55:52 wasowski Exp
*)

structure AbsynLogic :> AbsynLogic = struct

  open AbsynPrint (* opens Absyn + used for some debug output *)
  infixr 0 $
  infix 9 sub

  val all = List.all

  type 'a array = 'a Array.array

  (* Wuddy package interfacing *)

  structure W = WuddySys
  
  type bdd       = W.bdd
  type varSet    = W.varSet
  val sprMapping = W.sprMapping

  fun sprSolution (pos,neg) :string = 
	let val p = String.concat $ Typeset.join_blanks(map Symbol.name pos) []
	    val n = String.concat $ Typeset.join_blanks(map Symbol.name neg) []
	in String.concat ["POS(",p,") NEG(", n, ")"] end

  (* returns the number of solutions as an integer - you should take care
     that there is no overflow. Returns the number of satisfying assignments
     only for the support. A bit around the world but I do not have anything
     else. 
     
     This is only for debugging purposes! Does not work for arbitrary big
     spaces. *)

  fun satcount_supp (r :bdd) :int =
	let val n = W.getVarnum() - (Vector.length $ W.scanset $ W.support r)
	    fun res x 0 = x
	      | res x n = res (x/2.0) (n-1)
	in Real.floor $ res (W.satcount r) n end 
	handle Overflow => ~1




  (* below are three forcing functions. Forcing takes a bdd describing a set
     of configurations and returns a bdd describing a subset which
     additionally satisfies given list of positive and negative conditions.
     Note that this is implemented using bdd.AND, not restrict. Initial
     implementation with restrict was wrong. There is a subtle difference
     between and and restrict in bdds. Both operations would result in the
     same set of solutions (modulo number of variables). The difference is
     that restrict eliminates variables from bdd (and its support), while AND
     forces them to be true (and they remain in support). We need sometimes to
     refer to positive conditions (for instance when a transition goes down
     the hierarchy), so this variables should still be present in the solution
     set. This is way AND is used. AW 6/07/2002 *)

  (* variables not in the domain will be ignored *)

  fun force (r :bdd) (pos :stateid list, neg :stateid list) :bdd =
	W.AND(W.build_guard (pos,neg), r) 
   




end
