/* File : /home/cvs/cvsroot/projects/visualstate/scope/CodGenC1.c,v 
 * Comment: Runtime interpreter for programs generated with CodGenC1
 * Release: scope-0-16_final
 * 
 * Copyright (C) 2001-2004 Andrzej Wasowski
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * NOTES:
 * 
 * Currently if guards are not used (SCOPE_GUARDS undefined) we only
 * exclude some construct to ensure type correctness (i.e. we don't refer
 * to dispatcher which is not generated in such case. However, we still
 * keep some code related to guards because it depends to strongly on the 
 * way we handle actions (UNMANGLE_ACGD). If this is dependent on using
 * guards then runtime-representation could also benefit (if guards are
 * absent). I do not think you would save even a single byte in trans
 * array (absent guards do not take place anyway) but the interpretation
 * would become simpler.
 * 
 * CodGenC1.c,v 1.80 2004/01/26 12:11:40 wasowski Exp 
 */

/**
 * a hack to compile an avr version easily 
 * (unfortunately no assert in avr lib, at least in the one I got installed)
 */

#ifdef NDEBUG
#define assert(p)
#else
#include <assert.h>
#endif


#define SCOPE_VERBOSE 0

#if (SCOPE_VERBOSE && !defined(NDEBUG))
	#define VERBOSE(msg)		{ printf ( "%s", msg ); }
	#define VERBOSE_OR(msg,s)	{ printf( "%s", msg ); \
					  print_or_sym(s); \
					  printf( "\n" ); }
	#define VERBOSE_AND(msg,s)	{ printf( "%s", msg ); \
					  print_and_sym(s); \
					  printf( "\n" ); }
#else
	#define VERBOSE(msg)
	#define VERBOSE_OR(msg,s) 
	#define VERBOSE_AND(msg,s)
#endif

#ifdef SCOPE_CONFSET

/* next_conf size */
static confiter neco_size = 0;
/* prev_conf size */
static confiter prco_size = 0;

#endif

/* targets size */
static tgtsiter tgts_size = 0;

#ifndef NDEBUG

#include <stdlib.h>
#include <stdio.h>

/* check if targets are empty (used in assertions) */
static int targets_empty(void)
{
	return tgts_size == 0;
}

/* check if conf is empty (used in assertions) */
static int conf_empty(conf c)
{

#ifdef SCOPE_CONFSET
	if (c == next_conf)
		return (neco_size == 0);
	else
		return (prco_size == 0);
#else
	int i = 0;

	for (; i < STATE_WIDTH; i++)
		if (c[i] != STMRK)
			return 0;
	return 1;
#endif

}

/* check if state s is a member of c */
int conf_member_ext(conf c, andstref s)
{

#ifdef SCOPE_CONFSET
	confiter i = 0;
	confiter size = (c == next_conf) ? neco_size : prco_size;

	for (; i < size; ++i)
		if (c[i] == s)
			return 1;
	return 0;
#else
	return c[*(GET_AND(s))] == s;
#endif

}

void next_conf_reset(void)
{

#ifdef SCOPE_CONFSET
	neco_size = 0;
#else
	confiter i;

	for (i = 0; i < STATE_WIDTH; ++i)
		next_conf[i] = STMRK;
#endif

}

int compar(const void *a, const void *b)
{
	if (*(andstref *) a < *(andstref *) b)
		return -1;
	else if (*(andstref *) a == *(andstref *) b)
		return 0;

	return 1;
}

void print_and_sym(andstref s)
{
	int c = 0;

	while (1) {
		if (and_syms[c].id == s) {
			printf("%s ", and_syms[c].name);
			break;
		} else ++c;
	}
}

void print_or_sym(orstref s)
{
	int c = 0;
	while (1) {
		if (or_syms[c].id == s) {
			printf( "%s", or_syms[c].name);
			break;
		} else ++c;
	}
}

/* check if what is an andstate. Only works in --debug mode, bcs it uses
 * symbolic dictionaries 
 */

unsigned is_andstate(andstref what)
{
	int c = 0;

	while (and_syms[c].id != 0) {
		if (and_syms[c].id == what)
			return 1;
		c++;
	}
	return 0;
}

/*
 * check if what is an orstate. Only works in --debug mode, bcs it uses
 * symbolic dictionaries 
 */

unsigned is_orstate(orstref what)
{
	int c = 0;

	while (or_syms[c].id != 0) {
		if (or_syms[c].id == what)
			return 1;
		c++;
	}
	return 0;
}

void dump_conf()
{
	int i = 0;
	conf myconf;

#ifndef SCOPE_CONFSET
	int neco_size = STATE_WIDTH;
#endif
	for (i = 0; i < neco_size; ++i)
		myconf[i] = next_conf[i];

	printf("conf: [");
	qsort(myconf, neco_size, sizeof(andstref), compar);
	for (i = 0; i < neco_size; ++i)
		if (myconf[i] != STMRK)
			print_and_sym(myconf[i]);
	printf("]\n");
}

VS_BOOL activeand_next(const andstref what)
{
#ifdef SCOPE_CONFSET
	andstref min = *(GET_AND(what) + 1);
	confiter i = 0;

	assert(min > STMRK);

	/* you should never ask for sthg which definitely is not a state */
	assert(what != STMRK);
	assert(what != HMRK);
	assert(what != IMRK);
	assert(is_andstate(what));
	/* you should never need to test activity of rootstate */
	assert(*GET_AND(what) != STMRK);
	/* or in other words (should be a redundant test): */
	assert(what != ROOT_STATE);
	/* non-state will never cause activity */
	assert(STMRK < what);

	for (; i < neco_size; ++i)
		if (next_conf[i] >= min && next_conf[i] <= what)
			return 1;
	return 0;
#else
	assert(what != ROOT_STATE);
	assert(*GET_AND(what) < STATE_WIDTH);
	return next_conf[*GET_AND(what)] == what;
#endif
}


VS_BOOL activeor_next(const orstref what)
{
#ifdef SCOPE_CONFSET
	/* inefficient but ok for testing */
	const horcell * i = GET_OR(what);	
	do {
		if ( activeand_next(*i) ) return 1;
		i++;
	} while ( *(i-1) < *i );

	return 0;
#else
	assert( what < STATE_WIDTH );
	return next_conf[what] != STMRK;
#endif
}



#endif	/* NDEBUG */




VS_BOOL activeand(const andstref what)
{
#ifdef SCOPE_CONFSET
	andstref min = *(GET_AND(what) + 1);
	confiter i = 0;

	assert(min > STMRK);

	/* you should never ask for sthg which definitely is not a state */
	assert(what != STMRK);
	assert(what != HMRK);
	assert(what != IMRK);
	assert(is_andstate(what));
	/* you should never need to test activity of rootstate */
	assert(*GET_AND(what) != STMRK);
	/* or in other words (should be a redundant test): */
	assert(what != ROOT_STATE);
	/* non-state will never cause activity */
	assert(STMRK < what);

	for (; i < prco_size; ++i)
		if (prev_conf[i] >= min && prev_conf[i] <= what)
			return 1;
	return 0;
#else
	return prev_conf[*GET_AND(what)] == what;
#endif
}

#ifdef SCOPE_SIGNALQUEUE

/* Signal Queue - implemented as a round buffer */
static queueiter qbeg = 0;
static queueiter qend = 0;

/*
 * silently assume that queue is long enough! which shall be the case,
 * since verification returns this result 
 */
void enqueue1(eventtag e)
{
	assert(e < EVENTNO);

	queue[qend] = e;

	if (qend == QUEUE_LENGTH - 1)
		qend = 0;
	else
		qend++;

	assert(qend != qbeg);
}

#ifdef SCOPE_SLIST
void enqueueList(signalref ref)
{
	while (signals[ref] != NOEVENT)
		enqueue1(signals[ref++]);

}
#endif

/*
 * returning NOEVENT means that the queue is empty result is returned in
 * CurrEvent.tag (global variable) That is the way you do it in this funny 
 * language to conserve space ... 
 */
void dequeue1(void)
{

	if (qend != qbeg) {	/* non empty */

		CurrEvent.tag = queue[qbeg];
		if (qbeg == QUEUE_LENGTH - 1)
			qbeg = 0;
		else
			qbeg++;
	} else
		CurrEvent.tag = NOEVENT;
}

#endif	/* SCOPE_SIGNALQUEUE */




/*************************************
 * Implementation of The Interpreter *
 *************************************/




/*
 * entering states. Invariant is that if called on the whole hierarchy
 * leaves vector of target empty. If called on the part removes the states
 * belonging to this part. This is deep entering, so it sometimes has to
 * search through the hierarchy to decide if the state should be entered. 
 */

static void enterand(andstref scope);

static void enteror(orstref scope)
{
	tgtsiter j = 0;
	const    horcell *i = GET_OR(scope);
	andstref min = *(GET_AND(*i) + 1);
	const    horcell *max = i;
	andstref andst = 0;

	assert(!activeor_next(scope));
	assert(*i != STMRK);
	VERBOSE_OR( "Entering ", scope );

	while (*max < *(max + 1))
		++max;

	assert(*max != STMRK);
	assert(*max != HMRK);
	assert(is_andstate(*max));
	assert(is_andstate(min));

	for (; j < tgts_size; ++j)

		if (targets[j] <= *max && targets[j] >= min) {
			for (; *i < targets[j]; ++i);
			assert(i <= max);
			andst = *i;
			assert(is_andstate(andst));
			break;
		}

	if (andst) {
		if (targets[j] == andst)
			targets[j] = targets[--tgts_size];
	} else {
		/* enter by default: history or initial */
#ifdef SCOPE_HISTORY
		if (*(max + 1) == HMRK)
			andst = history[*(max + 2)];
		else
#endif
			andst = *GET_OR(scope);
	}
	exec(GET_ENTER(andst));
#ifndef SCOPE_CONFSET
	assert (andst != STMRK);
	assert (andst != ROOT_STATE);
	assert (is_andstate(andst));
	assert (scope < STATE_WIDTH);
	next_conf[scope] = andst;
#endif
	enterand(andst);

	assert( activeor_next(scope) );
}





static void enterand(andstref scope)
{
	const handcell *i = GET_AND(scope) + 1;

	assert(is_andstate(scope));
	VERBOSE_AND( "Entering ", scope );

	if (*i != scope) {
		++i;
		do {
			enteror(*i);
			++i;
		}
		while (*(i - 1) < *i);
	}
#ifdef SCOPE_CONFSET
	else {
		/* add scope to next configuration */
		assert(neco_size < STATE_WIDTH);
		next_conf[neco_size++] = scope;
	}
#endif
	assert(scope == ROOT_STATE || activeand_next(scope));

}

#ifdef IMPURE_EXIT

static void exitor_impure(orstref);

static void exitand_impure(andstref s)
{
	const handcell *i = GET_AND(s) + 1;

	VERBOSE_AND( "Exiting-and ", s );

	assert(is_andstate(s));
	assert(activeand_next(s));

	if (*i != s) {
		++i;
		do {
			exitor_impure(*i);
			++i;

		} while (*(i - 1) < *i);
	}
#ifdef SCOPE_CONFSET
	else {
		/* remove s from configuration */
		confiter c = neco_size - 1;
		assert(conf_member_ext(next_conf, s));

		for (; next_conf[c] != s; --c) {
			assert(c > 0);
		}
		next_conf[c] = next_conf[--neco_size];
	}
	assert(!activeand_next(s));
#endif	/* SCOPE_CONFSET */
}

#else	/* not IMPURE_EXIT */

#define exitor_impure(s)	exitor_pure(s)

#endif





#ifdef IMPURE_EXIT

static void exitor_impure(const orstref scope)
{
#ifdef SCOPE_CONFSET
	confiter j;
	const horcell *i = GET_OR(scope);
	andstref min = *(GET_AND(*i) + 1);
	const horcell *max = i;
	confiter oldbufsize = neco_size;
#endif
	andstref andst;

	VERBOSE_OR("Exiting-or-impure ", scope );
	assert( activeor_next(scope) );

#ifdef SCOPE_CONFSET
	while (*max < *(max + 1))
		++max;
	assert(*i != STMRK);
	assert(*max != STMRK);
	assert(*max != HMRK);
	assert(is_andstate(*max));
	assert(is_andstate(min));

	for (j = 0; j < neco_size; ++j)

		if (next_conf[j] <= *max && next_conf[j] >= min) {
			for (; *i < next_conf[j]; ++i);
			assert(i <= max);
			andst = *i;
			break;
		}

#if defined(SCOPE_HISTORY) 
	if (*(max + 1) == HMRK)
		history[*(max + 2)] = andst;
#endif
	
#else 
	assert (scope < STATE_WIDTH);
	andst = next_conf[scope];

#if defined(SCOPE_HISTORY) 
	{
		const horcell *i = GET_OR(scope);
		do ++i; 
		while (*(i-1) < *i);
		if (*i == HMRK)
			history[*(i + 1)] = andst;
	}
#endif

#endif


	assert(andst != ROOT_STATE);
	assert(andst != STMRK );
	assert(andst != HMRK );
	assert(is_andstate(andst));
	assert(activeand_next(andst));

	exitand_impure(andst);

#ifndef SCOPE_CONFSET /* remove from next configuration */
	assert(scope < STATE_WIDTH);
	next_conf[scope] = STMRK;
	assert(!activeand_next(andst));
#endif
	exec(GET_EXIT(andst));
#ifdef SCOPE_CONFSET
	assert(oldbufsize >= neco_size);
#endif
	assert(!activeor_next(scope));

	VERBOSE_OR("Exited-or-impure ", scope );

} /* exitor_impure() */

#endif				/* IMPURE_EXIT */

#ifndef PURE_EXIT

#define exitor_pure(s) exitor_impure(s)

#else				/* defined(PURE_EXIT) */

#ifndef SCOPE_CONFSET
#error "Pure exit only works in confset mode"
#endif 

void exitor_pure(orstref s)
{
#ifndef NDEBUG
	int removed = 0;
	int presize = neco_size;
#endif
	confiter j = 0;
	const andstref *const i = GET_OR(s);
	andstref min = *(GET_AND(*i) + 1);
	const horcell *max = i;

	VERBOSE_OR( "Exiting-or-pure ", s );
	assert(is_orstate(s));
	assert(!conf_empty(next_conf));
	assert(activeor_next(s));

	while (*max < *(max + 1))
		++max;

	assert(is_andstate(*max));
	assert(ROOT_STATE > *max || ROOT_STATE < min);

	while (j < neco_size)
		if (next_conf[j] >= min && next_conf[j] <= *max) {
			next_conf[j] = next_conf[--neco_size];
#ifndef NDEBUG
			removed++;
#endif
		} else
			j++;

	assert(removed);
	assert(neco_size + removed == presize);
	assert(!activeor_next(s));
}

#endif				/* PURE_EXIT */

/* Fire a transition. Transition is assumed to be enabled.  The parameter
 * passed is assumed to be the index in transitions array were the
 * guard/action indicators are kept. We point into the middle of the
 * record for slight efficiency gain. The preceding part will not be used
 * in this function anyway. The assumption is that targets are empty on
 * entry and on exit. 
 */

void fire(const transcell * ptr)
{
	actionref ac = STMRK;
	const transcell *s;

#ifdef SCOPE_GUARDS
	guardref gd = STMRK;

	UNMANGLE_ACGD(ptr, ac, gd);
	if (!eval(gd))
		return;
#else
	UNMANGLE_AC(ptr, ac);
#endif
	assert(targets_empty());

	s = ptr;

	while (*s != STMRK) {

		assert(*s < MARKNO);

#ifndef IMPURE_EXIT
		assert(*s != IMRK && *s != HMRK);
#endif
#ifndef PURE_EXIT
		assert(*s != FPSCOP && *s != NFPSCOP);
#endif

		switch (*(s++)) {
#ifdef IMPURE_EXIT
		case HMRK:	/* flat non pure exit */

			do
				exitor_impure(*(GET_AND(*(s++))));
			while (*s >= MARKNO);
			break;

		case IMRK:	/* nonflat and nonpure exit */

			exitor_impure(*(s++));
			while (*s >= MARKNO)
				s++;
			break;
#endif
#ifdef PURE_EXIT
		case FPSCOP:	/* flat pure exit */

			do
				exitor_pure(*(GET_AND(*(s++))));
			while (*s >= MARKNO);
			break;

		case NFPSCOP:	/* nonflat and pure exit */

			exitor_pure(*(s++));
			while (*s >= MARKNO)
				s++;
#endif
		}

		assert(*s < MARKNO);
	}
	
	VERBOSE("TRANAC:")
	exec(ac);
	VERBOSE(" TRANAC\n")

	while (*ptr != STMRK) {

		assert(*ptr < MARKNO);

#ifndef IMPURE_EXIT
		assert(*s != IMRK && *s != HMRK);
#endif

#ifndef PURE_EXIT
		assert(*s != FPSCOP && *s != NFPSCOP);
#endif

		if (
#ifdef IMPURE_EXIT
			   *ptr == HMRK ||
#endif
#ifdef PURE_EXIT
			   *ptr == FPSCOP
#else
			   0
#endif
		    ) {		/* flat */

			while (*(++ptr) >= MARKNO) {
				targets[0] = *ptr;
				tgts_size = 1;
				enteror(*(GET_AND(*ptr)));
			}

		} else {	/* non flat */

			orstref scope = *(++ptr);

			for (++ptr; *ptr >= MARKNO; ++ptr)
				targets[tgts_size++] = *ptr;
			enteror(scope);
		}

		assert(targets_empty());
		assert(*ptr < MARKNO);
	}
}

/* microstep - assumes that proper event is already in CurrEvent no
 * conflict resolution currently. Everything will collapse on conflict 
 */

void microstep(void)
{

	const transcell *t = trans + tranidx[CurrEvent.tag];

	assert(CurrEvent.tag < NOEVENT);

	if (tranidx[CurrEvent.tag] != TRANS_MAX)
		do {

			unsigned int pc, nc;

			UNMANGLE_PCNC(t, pc, nc);

			for (; pc > 0; --pc, ++t)
				if (!activeand(*t))
					goto skiptr;

			for (; nc > 0; --nc, ++t)
				if (activeand(*t))
					goto skiptr;

			fire(t);	/* checking guard is deferred to fire */

		      skiptr:while (*t != STMRK)
				++t;
			++t;

		}
		while (t < trans + tranidx[CurrEvent.tag + 1]);
}

/* assumes that external event is already in CurrEvent the newest
 * configuration is always kept in next_conf. prev_conf is only occasionally
 * used for guard computations. 
 */
void macrostep(void)
{

#ifdef SCOPE_SIGNALQUEUE
	while (CurrEvent.tag != NOEVENT) {
#endif
		confiter i = 0;
#ifdef SCOPE_CONFSET
		prco_size = neco_size;
		for (; i < neco_size; ++i)
			prev_conf[i] = next_conf[i];
#else
		for (; i < STATE_WIDTH; ++i)
			prev_conf[i] = next_conf[i];
#endif

		assert(!conf_empty(next_conf));

		microstep();

#ifdef SCOPE_SIGNALQUEUE
		dequeue1();
	}
#endif

}

/* initialise the model Note: st_init will only work on empty configuration.
 * If there is a need for multiple initializations at runtime another
 * function (see testdrv) has to be called to clean current conf 
 */

void st_init(void)
{
#ifdef SCOPE_CONFSET
	prco_size = neco_size = 0;
#else
	confiter i = 0;
	for( i; i < STATE_WIDTH; ++i ) 
		prev_conf[i] = next_conf[i] = STMRK;
#endif
	enterand(ROOT_STATE);

#ifdef SCOPE_SIGNALQUEUE
	dequeue1();
	macrostep();
#endif
}
