(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/CodGenC1_front.sml,v
   Comment: Front end of CodGenC1 code generator. Takes an Absyn model and
	    compiles it to lrsystem (intermediate representation)
   Release: scope-0-16_final 

     Copyright (C) 2001-2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   NOTES:

   This modules tries to avoid operating on numbers (addresses, references,
   etc). Symbolic names are used as far as possible and actual array
   addresses are assigned in scheduler. One exception is actions and guards,
   as we need to count the number of unique before type inference. This is
   conveniently done by constructing an ulist, which performs a scheduling in
   place.

   CodGenC1_front.sml,v 1.53 2003/07/24 11:27:42 wasowski Exp
*)


(* some interface *)

open List CodGenC1_LR CodGenC_front Absyn
infixr 1 $; 
infix 3 --;

val alloc = UList.alloc

type actions = CodGenC1_LR.actions


(* Types *)

(* adressing mode selector for targets with scopes *)

fun isflat ([s1],s2) :bool = parentand s1 = nameor s2
  | isflat _ = false  

(* FIXME: this function is whole lot too
   ugly. It calls for rethinking and reimplementation. 
   
   mode saves information on which addresing modes have been used,
   so they should be activeated later on *)

fun comp_targets confset []      prev (res,mode) :lratom list * exitmode = (res,mode)
  | comp_targets confset (t::ta) prev (res,mode) =	
	let val _ = assert (fn()=> not (null (#1 t))) 
			"#200307150:Empty targets group!"
	in case (isflat t,expureor (#2 t) andalso confset) of 
			(* not confset means that exitor is not *)
			(* allowed in other modes               *)

		  (true,false) => 
			if prev = HMRK then 
			
				comp_targets confset ta HMRK 
				((LRAndStateRef $ nameand $ hd $ #1 t)::res, 
					use_impure mode)
			else 
				comp_targets confset ta HMRK 
				((LRAndStateRef $ nameand $ hd $ #1 t)::LRMrk HMRK ::res, 
					use_impure mode)
		| (false,false) =>
			comp_targets confset ta IMRK
			(foldl (fn (s,e) => LRAndStateRef(nameand s)::e) 
			((LROrStateRef $ nameor $ #2 $ t) :: LRMrk IMRK::res) (#1 t), use_impure mode)
		| (true,true) =>
			if prev = FPSCOP then 
				comp_targets confset ta FPSCOP 
				((LRAndStateRef $ nameand $ hd $ #1 t)::res,use_pure mode)
			else 
				comp_targets confset ta FPSCOP 
				((LRAndStateRef $ nameand $ hd $ #1 t) :: LRMrk FPSCOP ::res,use_pure mode)
		| (false,true) =>
			comp_targets confset ta NFPSCOP 
			(foldl (fn (s,e) => LRAndStateRef(nameand s)::e)
			((LROrStateRef $ nameor $ #2 $ t) :: LRMrk NFPSCOP ::res) (#1 t),use_pure mode)
	end
			
(* compile a single transition, suitable for folding *)

fun comp_tran (h :attr hierarchy) (confset :bool) (ets :evtype env) 
	(t :static tran, (tenv :tenv, guards, actions :actions, mode)) 
		:tenv * guards * actions * exitmode =
	let val subst = comp_subst ets (getebind (#trigger t))
	    val (actref,actions1) = 
			alloc (subst_action subst (#action t),actions)
	    val guard = if null (#guard t) then always else (#guard t)
	    val (guardref,guards1) =
			alloc (subst_guard subst guard, guards)
	    val stargets = map (fn{stsrcs,stscope,tgts}=> 
				(map (getandh h) tgts,getorh h stscope)) 
					(#annotation t)
	    val (targets,mode1) = comp_targets confset stargets NFPSCOP ([],mode)
	    val trans  = SE.lookup tenv (#1 $ getebind $ #trigger t) 
				handle NotFound => [] 
	    val trans1 = List.revAppend (targets, LRMrk STMRK::trans)
	    val trans2 = LRAcGd (actref+markno,guardref+markno) ::trans1
	    val trans3 = map LRAndStateRef (#pos t @ #neg t) @ trans2
	    val trans4 = LRCondCounters(length (#pos t), length (#neg t))::trans3
	    val tenv1  = SE.bind1 tenv (#1 $ getebind $ #trigger t, trans4)
	in (tenv1, guards1, actions1,mode1) end
	handle NotFound => impossible "#20020709b"

(* compile a transitions list (already degrouped, no dynmic scopes) *)

fun comp_trans (h :attr hierarchy) (confset :bool) (ets: evtype env) 
		(actions :actions) (trans :static tran list) 
			:tenv * guards * actions * exitmode =
	let val guards = #2 (alloc (always,UList.initialize guardsize))
	    val nomode = { pure = false, impure = false }
	in foldl (comp_tran h confset ets) (empty,guards,actions,nomode) trans end
	handle NotFound => impossible "#20020709a"

(* compiling states. functions should be used with hfolds *)

fun compor (ORState s, (hor,actions,history))
	:lratom list * actions * lratom list =
	let val kids    = map nameand (#children s)
	    val (hor1,history1) = 
			if #marking s <> History then (hor,history)
			else (LRMrk HMRK ::LRHistoryRef (length history) ::hor,
			      LRAndStateRef (hd kids) ::history)
	    val hor2    = map LRAndStateRef kids @ hor1
	    val hor3    = LRStateLabel(#name s) :: hor2
			  handle Empty => intErr Empty 200210290
	in (hor3, actions, history1) end
	

fun compand (ANDState s, (hand,actions)) : lratom list * actions  = 
	let val (enref,actions1) = alloc (#entry s,actions) 
	    val (exref,actions2) = alloc (#exit s,actions1)
	    val hand1 = foldl (fn (s,e) => LROrStateRef (nameor s)::e) 
				hand (#children s)
	    val hand2 = (LRAndStateRef $ nameand $ LMDand $ ANDState s) ::hand1
	    val hand3 = LROrStateRef (valOf $ #parent s) ::hand2
			handle Option.Option => LRMrk STMRK ::hand2
	    val hand4 =	LREnEx (enref+markno,exref+markno) 
				::LRStateLabel (#name s) ::hand3

	    (* +(markno) below reserves space for markers in action table *)
	    val hand5 = if (enref+markno <> null_actref 
				orelse exref+markno <> null_actref)
			then LRMrk STMRK :: hand4 else hand4
	in (hand5,actions2) end 


(* compile hierarchy. result is: hor and hand arrays in lr format. list of
   actions in internal representation, history state vector in lratom format *)

fun comp_states (h :attr hierarchy) 
	:lratom list * lratom list * actions * lratom list =
	let val actions = #2 (alloc (null_action,UList.initialize actionsize))
	    val history = [LRMrk STMRK]
	    fun id (x,e) = e
	    (* folds are reversed comparing the paper since in functional *)
	    (* language it is more handy to generate list from the end    *)
	    val (hor, actions,history) = hfoldland compor  id 
				([LRMrk STMRK],actions,history) (gettree h)
	    val (hand,actions)         = hfoldrand id compand 
				([LRMrk STMRK],actions) (gettree h)
	    val hand1   = List.tabulate(markno, fn _ => LRMrk HMRK) @ hand
	    val hor1    = List.tabulate(2,      fn _ => LRMrk HMRK) @ hor
	in  (hor1, hand1, actions,history) end

	
(* compile system, compute some sizes and infer types *)

fun comp_system (queuelen :int,confset :bool) (sys :(attr,static) system) :lrsystem =
	let val (_,h) = Normalise.pull_defs (#denv sys, #hierarchy sys)
	    val (hor,hand,actions1,history) = comp_states h 
	    val (tenv,guards,actions2,exitmode)= 
			comp_trans (#hierarchy sys) confset 
				(#evtypes sys) actions1 (#trans sys) 
		(* safe to call on non transformed system *)
	    val (orsts,andsts) = Stat.state_dist sys
	    val _ = assert (fn ()=> orsts < andsts)
			"#200307151:|State_or| >= |State_and|"
	in { name     = Symbol.name (#name sys),
	     fname    = Symbol.name (#fname sys),
	     hor      = hor,
	     hand     = hand,
	     root     = nameand (gettree h),
	     tenv     = tenv,
	     
	     history  = history,
	     signals  = comp_signals actions2,
	     actions  = actions2,
	     guards   = guards,

	     evtypes  = #evtypes sys,
	     vartypes = #vartypes sys,
	     venv     = #venv sys,
	     cenv     = #cenv sys,

	     queuelen = comp_queuelen queuelen (#evtypes sys),
		(* safe to call on non transformed system *)
	     statew   = if confset then 
				Stat.conf_card_bound sys
			else (orsts + 2),
	     targetw  = Stat.targets_card_bound (#trans sys),
	     orsts    = orsts,
	     andsts   = andsts,
	     exitmode = exitmode }
	end

(* infer type sizes for given system in LR *)

fun resolve_types (sys :lrsystem) :lrsizes = 
	let val evnumber  = SymbolEnv.count (#evtypes sys)
	    val signumber = UList.length (#signals sys)
	in { 
		actionref = Types.mintysize (UList.length(#actions sys) + markno -1),
		guardref  = Types.mintysize (UList.length(#guards  sys) + markno -1),
		signalref = Types.mintysize (evnumber + 1 + signumber - 1),
		(* signalrefsize can be done better, substracting 
		   the length of the last entry *)
		orstref   = mintysize (#orsts sys + 1 - 1),
		andstref  = mintysize (#andsts sys+ markno - 1),
		evnumber  = evnumber }
	end



