(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/CodGenHsm.sml,v
   Comment: Dumps hierarchy structure to hsm file (simplified hierarchical
            state machine). THIS IS NOT SEMANTICS PRESERVING 
   Release: scope-0-16_final 

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   CodGenHsm.sml,v 1.5 2003/07/08 15:45:01 wasowski Exp
*)

structure CodGenHsm : CODGEN = struct

open Absyn CodGen Cppish Err Say Typeset
infix 0 << <<< <<> <- (* some fun with C++ style IO *)
infix 3 --

structure H  = Hsm
structure S  = Symbol
structure SE = SymbolEnv

type options = string option (* output file name *)
type aor = unit
type atr = unit

fun isdot c = c = #"."

fun trim statename = List.last (String.tokens isdot statename)

fun trim2scope (scopepath :string list) (statename :string) :string list =
	let val _	  = assert (fn ()=> scopepath<>[]) "#200305010"
	    val _	  = assert (fn ()=> length scopepath > 1) "#200305014"
	    val statepath = String.tokens isdot statename
	    val _	  = assert (fn ()=> statepath<>[]) "#200305011"
	    val prefix    = List.take (statepath, length scopepath)
	    val _         = assert (fn()=> prefix = scopepath) "#200305012"
	    val suffix    = List.drop (statepath, length scopepath)
	    val _         = assert (fn()=> not (null suffix)) "#200305013"
	in suffix end

fun tran2hsm scope ({ trigger, pos, action, targets, ... } :atr tran) :H.tran = 
	let val event = case trigger of Event(e,_) => S.name e 
				      | EventGroup e => S.name e
	    val scopepath = String.tokens isdot scope 
	    fun getac [] = []
	      | getac ((Expr (Call (ou,_)))::tail) = S.name ou :: getac tail
	      | getac (_::tail) = getac tail
	    val source = trim2scope scopepath (S.name (hd pos))(* unsafe *)
	    val target = trim2scope scopepath (S.name (hd targets)) (* unsafe *)  
	in { event = event,
	     action= getac (#1 action),
	     source= source, 
	     target= target }
	end
	

fun andstate2hsm (denv :denv) trans (default :bool,ANDState {name,children,...})
		:H.andstate = H.AND (trim (S.name name), 
			      default, map (orstate2hsm denv trans) children)

and orstate2hsm (denv :denv) trans (s as ORState {name,children,...}) 
								:H.orstate =
	let val default       = SE.lookup denv name
	    val paired        = map (fn s => (nameand s=default,s)) children
	    val (here,trans1) = List.partition (fn t=> #scope t = name) trans
	    val kids          = map (andstate2hsm denv trans) paired
	    val scope         = S.name name
	in H.OR (trim (S.name name), kids, map (tran2hsm scope) here) end

fun scope2hsm (sys : (aor,atr) system) : H.hsm = 
	let val events = map (S.name o #1) (SE.toList (#evtypes sys))
		       @ map (S.name o #1) (SE.toList (#evgroups sys))
	    val actions = map (S.name o #1) (SE.toList (#otypes  sys))
	in { 
		events   = Listsort.sort String.compare events,
		actions  = Listsort.sort String.compare actions,
		hierarchy= andstate2hsm (#denv sys) (#trans sys) 
				(true,#1 (#hierarchy sys))
	} end

fun compile1 (sys :(aor,atr) system) :codgen = HsmPrint.print (scope2hsm sys)

fun compile (ofile :string option) (ifile :string) (m :(aor,atr) model) :unit =
	let val file = case ofile of NONE => Path.file ifile ^ ".hsm"
				    | SOME f => f
	    val os = TextIO.openOut file
	in 
		os << "/* Automatically generated by SCOPE from: " << ifile
		   << " */\n\n";

		foldl (fn (s,os) => os <- compile1 s << "\n") os m;
		TextIO.closeOut os
	end

	

end


