/* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/VSTypes.h,v
   Comment: Wrapper for stdint.h to provide VS state like names ints with fixed sizes.
   Release: scope-0-16_final

     Copyright (C) 2001-2004 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   NOTES:

   In VS ver. 4.x there is a file with the same names which implements the same types
   without using stdint.h. This implementation was made to avoid copyright issues with 
   IAR and also because I think it is generally a good idea to use things provided by 
   ISO C99 standard.

   This will not work if your C compiler does not provide the types (or does not provide 
   typedefs for the types)

   VSTypes.h,v 1.9 2004/05/13 15:26:30 wasowski Exp
*/

#ifndef _scope_VSTYPES_H
#define _scope_VSTYPES_H

#if (__STDC_VERSION__ > 199901)

#include <stdint.h> /* C99 type definitions */

#define VS_TRUE  '\1'
#define VS_FALSE '\0'

typedef unsigned char	VS_BOOL;
typedef unsigned char	VS_UCHAR;
typedef signed char	VS_SCHAR;
typedef unsigned int	VS_UINT;
typedef signed int	VS_INT;
typedef float		VS_FLOAT;
typedef double		VS_DOUBLE;
typedef void		VS_VOID;
typedef void *		VS_VOIDPTR;

#ifdef INT8_MAX
typedef uint8_t		VS_UINT8;
typedef int8_t		VS_INT8;
#endif

#ifdef INT16_MAX
typedef uint16_t	VS_UINT16;
typedef int16_t		VS_INT16;
#endif

#ifdef INT32_MAX
typedef uint32_t	VS_UINT32;
typedef int32_t		VS_INT32;
#endif

#else




#include <limits.h>



#ifdef SE_EMBED
#define VS_FILE_TYPE void
#else
#define VS_FILE_TYPE FILE
#endif



#define VS_BOOL            unsigned char      
#define VS_UCHAR           unsigned char      
#define VS_SCHAR           signed char        
#define VS_UINT            unsigned int       
#define VS_INT             signed int         
#define VS_FLOAT           float              
#define VS_DOUBLE          double             
#define VS_VOID            void               
#define VS_VOIDPTR         void*              

#if (UCHAR_MAX >= 0x0ff)
#define VS_UINT8           unsigned char      
#define VS_INT8            signed char        
#elif (USHRT_MAX >= 0x0ff)
#define VS_UINT8           unsigned short     
#define VS_INT8            signed short       
#elif (UINT_MAX >= 0x0ff)
#define VS_UINT8           unsigned int       
#define VS_INT8            signed int         
#elif (ULONG_MAX >= 0x0ff)
#define VS_UINT8           unsigned long      
#define VS_INT8            signed long        
#else
#define VS_UINT8           (unsupported data type)
#define VS_INT8            (unsupported data type)
#endif

#define VS_UINT16_VAARG    VS_INT
#define VS_INT16_VAARG     VS_INT

#if (UCHAR_MAX >= 0x0ffff)
#define VS_UINT16          unsigned char      
#define VS_INT16           signed char        
#elif (USHRT_MAX >= 0x0ffff)
#define VS_UINT16          unsigned short     
#define VS_INT16           signed short       
#elif (UINT_MAX >= 0x0ffff)
#define VS_UINT16          unsigned int       
#define VS_INT16           signed int         
#elif (ULONG_MAX >= 0x0ffff)
#define VS_UINT16          unsigned long      
#define VS_INT16           signed long        
#undef VS_UINT16_VAARG
#undef VS_INT16_VAARG
#define VS_UINT16_VAARG    VS_INT16
#define VS_INT16_VAARG     VS_INT16
#else
#define VS_UINT16          (unsupported data type)
#define VS_INT16           (unsupported data type)
#endif

#define VS_UINT32_VAARG    VS_INT
#define VS_INT32_VAARG     VS_INT

#if (UCHAR_MAX >= 0x0ffffffff)
#define VS_UINT32          unsigned char      
#define VS_INT32           signed char        
#elif (USHRT_MAX >= 0x0ffffffff)
#define VS_UINT32          unsigned short     
#define VS_INT32           signed short       
#elif (UINT_MAX >= 0x0ffffffff)
#define VS_UINT32          unsigned int       
#define VS_INT32           signed int         
#elif (ULONG_MAX >= 0x0ffffffff)
#define VS_UINT32          unsigned long      
#define VS_INT32           signed long        
#undef VS_UINT32_VAARG
#undef VS_INT32_VAARG
#define VS_UINT32_VAARG    VS_INT32
#define VS_INT32_VAARG     VS_INT32
#endif

#endif /* __STDC_VERSION__ */
#endif /* _scope_VSTYPES_H */
