(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/acfg/acfg-sig.sml,v
   Comment: Signature for implementations of acyclic control flow graphs
	    implementations.
   Release: scope-0-16_final

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   acfg-sig.sml,v 1.4 2003/01/22 16:08:08 wasowski Exp 
*)

signature ACFG = sig

	eqtype (''a,''b) test
	eqtype (''a,''b) acfg

	val actions : (''a,''b) acfg -> ''b list
	val tests   : (''a,''b) acfg -> (''a,''b) test list

	val cond    : (''a,''b) test -> ''a
	val hi      : (''a,''b) test -> (''a,''b) acfg
	val lo      : (''a,''b) test -> (''a,''b) acfg
	
	val is_leaf : (''a,''b) acfg -> bool

	val Test : ''a * (''a,''b) acfg * (''a,''b) acfg -> (''a,''b) test
	val ACFG : ''b list * (''a,''b) test list -> (''a,''b) acfg 
end

(*
	[seq] type of acyclic control flow graphs. Equality is demanded to
	help in maintaining reducedness in some implementations. Sequence is 
	a sequence of actions to execute and a sequence of splits (trees) to
	test sequentially.

	[is_leaf node] true for leaf node (i.e node with no more trees nested)

	[actions node] returns the list of actions assigned with this acfg 
	(i.e. actions to be executed prior or after traversing the list of
	trees)

	[cond node] returns the condition tested in the node. 

	[mk_split] constructs a branch node of acyclic control flow graph
	with given parameters. In tree implementation it will merely be
	renaming for a constructor but in reduced implementations this may
	perform a real work.

	[mk_seq] constructs a sequence node of acyclic control flow graph. A
	sequence node is a list of actions and list of trees to test on.
*)

