(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/common.sml,v
   Comment: Common types of flat and hierarchical models
   Release: scope-0-16_final

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   common.sml,v 1.8 2003/07/29 16:46:35 wasowski Exp
*)

structure Common = struct

open Types Exp Err

structure S  = Symbol
structure SE = SymbolEnv

type symbol = Symbol.symbol
type 'a env = 'a SymbolEnv.env
val  empty  = SE.empty

type event = symbol
type evgrpid = symbol
type stateid = symbol	(* Assumed to be a fully qualified state name 
			   so state symbol identifies state uniquely:
			   systemname.topstatename. .... .myname *)

type venv = value env

(* signal expression *)

type sexp = event * exp list		    (* name, [param] *)

(* actions *)

type action = command list * sexp list

type actions = action list

(* joins two actions. WARNING: the second param is appended
   IN FRONT of the first! *)

fun appendact (a1 : action) (a2 :action) :action = 
			(#1 a2 @ #1 a1, #2 a2 @ #2 a1)


fun signals (a :action) = #2 a
fun commands (a :action) = #1 a

val empty_action :action = ([],[])

type symbolset   = Set.set
type eventset    = symbolset

(* default state environment *)		  

type denv = symbol env

(* event bindings *)
  
type ebind = symbol * symbol list	    (* name, [param_list] *)

(* element types *)

type evgroup = eventset 
  
datatype trigger = EventGroup of symbol
		 | Event      of ebind 

type guard = bexp list

(* unboxing event trigger *)

fun getebind (Event eb :trigger) = eb 
  | getebind (EventGroup _) = impossible "Event expected, group found!"

fun getevent (Event eb :trigger) :symbol = #1 eb
  | getevent (EventGroup sg    ) :symbol = sg

fun bound_names (Event (e,ps) :trigger) = ps
  | bound_names _ = []

end
