(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tSpecialize.sml,v
   Comment: testcases for Specialize.sml
   Release: scope-0-16_final

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


   tSpecialize.sml,v 1.10 2003/07/08 15:45:06 wasowski Exp
*)

app load ["Main","tRLCommon","Specialize"];

structure tSpecialize : TEST_SET = struct

val name = "Specialize";

(* testcases *)

type atr = unit
type aor = unit

type input  = unit frag list
type output = (aor,atr) Absyn.model * (aor,atr) Absyn.model option		
		(* full_model, specialized *)
type testcase = string * input * (output -> bool)

(* helpers *)

fun isExpr (Expr _) = true
  | isExpr (Assgn _)= false

fun SUCCESS file rl accept = (file, rl, fn (p,n)=> accept(p,valOf n))
fun FAILURE file rl        = (file, rl, fn (p,n)=> not(isSome n)    )

val testcases : testcase list = [

		SUCCESS "test/run/spec01.vsp" `

			/* no transitions eliminated */
			restriction SPEC01 { impossible event3(); };
			SPEC01 spec01;
		
		`	(all2 (fn (s,p)=>(#trans s = #trans p))),

		SUCCESS "test/run/spec01.vsp" `

			/* eliminate one transition */
			restriction SPEC01 { impossible event1(); };
			SPEC01 spec01;
		
		`	(all2 (fn (s,p)=>
				(length(#trans s) > length (#trans p)
				 andalso not (null (#trans p))) )),
		
		SUCCESS "test/run/spec01.vsp" `

			restriction SPEC01 { impossible event2(); };
			SPEC01 spec01;
		
		`	(all2 (fn (s,p)=>
				(length(#trans s) > length (#trans p) 
				 andalso not (null (#trans p))) )),

		SUCCESS "test/run/spec01.vsp" `

			restriction SPEC01 { impossible event1(), event2(); };
			SPEC01 spec01;
		
		`	(all2 (fn (_,p)=> null (#trans p))),

		SUCCESS "test/run/spec01.vsp" `

			/* dead should remove all assignments */
			restriction SPEC01 { dead VS_INT extvar; };
			SPEC01 spec01;

		`	(all2 (fn (_,sys)=> 
				all (fn tran => all isExpr (#1(#action tran)))
				(#trans sys))
			),
		
		SUCCESS "test/run/spec01.vsp" `

			/* const should remove all assignments */
			restriction SPEC01 { const VS_INT extvar = 7; };
			SPEC01 spec01;

		`	(all2 (fn (_,sys)=> 
				all (fn tran => all isExpr (#1(#action tran)))
				(#trans sys))
			),
		
		SUCCESS "test/run/spec01.vsp" `

			/* should not remove assignments */
			restriction SPEC01 { };
			SPEC01 spec01;

		`	(all2 (fn (_,sys)=> 
				exists (fn tran => exists (not o isExpr) 
				    (#1(#action tran)))
			      (#trans sys))
			),

		SUCCESS "test/run/spec02.vsp" `

			restriction SPEC02 { impossible event1(), event3(); };
			SPEC02 spec02;

		`	(all2 (fn (_,r)=> Set.isEmpty (SE.lookup (#evgroups r)
						(S.symbol "Event1or3")))),
		SUCCESS "test/run/spec02.vsp" `

			/* Group Event1or3 should have 2 members */
			restriction SPEC02 {};
			SPEC02 spec02;
		
		`	(all2 (fn (_,r)=> Set.numItems (SE.lookup (#evgroups r)
						(S.symbol "Event1or3"))=2)),
		SUCCESS "test/run/spec02.vsp" `

			restriction SPEC02 { impossible event1(), 
							event3(); };
			SPEC02 spec02;
		
		`	(all2 (fn (_,sys) => Set.isEmpty (SE.lookup 
					(#evgroups sys)
					(S.symbol "Event1or3"))
					andalso
					Set.numItems (SE.lookup
					(#evgroups sys)
					(S.symbol "AnyEvent"))=1)),

		SUCCESS "test/run/spec03.vsp" `

			/* preserver transition triggered by signal */
			/* presrve action of triggering signal */
			restriction SPEC03 {};

			SPEC03 spec03;

		`	(all2 (fn (_,sys) => length (#trans sys)=3 andalso
				(exists (fn t=> not(null(#2(#action t)))) 
				(#trans sys)))),

		SUCCESS "test/run/spec03.vsp" `

			/* transition triggered by signal: remove */
			/* action of triggering signal: remove */
			restriction SPEC03 { impossible signal(); };

			SPEC03 spec03;

		`	(all2 (fn (_,sys) => length (#trans sys)=2 andalso
				(all (fn t=> null(#2(#action t))) (#trans sys)))),

		SUCCESS "test/run/spec03.vsp" `

			/* 12 spec variable rvalue */
			restriction SPEC03 {

				/* eliminate all but one transition */
				impossible signal(), event2();
				/* const sthg used rvalue */
				const VS_INT var2 = 9;
			};

			SPEC03 spec03;

		`	(all2 (fn (_,sys) => length (#trans sys) = 1
				andalso let val (c,s)= #action(hd(#trans sys))
				in 
				
				c = [Assgn(AccessVar(S.symbol "var1"),
					     Prim2(OpAdd, Cst (CstI 9), 
							  Cst (CstI 7)))]
				
				andalso (* external var remains as it was *)
				   not(SE.defined (#cenv sys) (S.symbol "var3"))
				
				andalso (* const remains a const *)
				   SE.defined (#cenv sys) (S.symbol "myconst")

				andalso (* var2 is undeclared *)
					not(SE.defined (#vartypes sys)
						(S.symbol "var2"))
				andalso not(SE.defined (#venv sys)
						(S.symbol "var2"))

				andalso (* var1, var3 are declared *)
					SE.defined (#vartypes sys)
						(S.symbol "var1")
				andalso SE.defined (#vartypes sys)
						(S.symbol "var3")
				andalso SE.defined (#venv sys)
						(S.symbol "var1")
				andalso SE.defined (#venv sys)
						(S.symbol "var3")
				end)),

		SUCCESS "test/run/spec03.vsp" `
		
			/* 13 spec variable lvalue and rvalue */
			restriction SPEC03 {

				/* eliminate all but one transition */
				impossible signal(), event2();
				/* const internal var in rvalue */
				const VS_INT var1 = 9;
				/* const an external var */
				const VS_INT var3 = 21;
			};

			SPEC03 spec03;

		`	(all2 (fn (_,sys) => length (#trans sys) = 1
				andalso let val c = #action(hd(#trans sys))
					    val g = #guard(hd(#trans sys))
					    val s = (hd o childrenor o hd o
							childrenand o #1 o
							#hierarchy) sys
				
				in (* actions in tran is as expected *)
				   c = ([Expr(Prim2(OpAdd, 
					Access(AccessVar(S.symbol "var2")), 
					Cst (CstI 7)))],[])

				andalso	(* actions in first state are ok *)
					enand s = ([Expr(Prim2(OpAdd, 
					Access(AccessVar(S.symbol "var2")), 
					Cst (CstI 5)))],[])
				andalso exand s = ([Expr(Prim2(OpAdd, 
					Access(AccessVar(S.symbol "var2")), 
					Cst (CstI 6)))],[])
					
				andalso (* guard is as expected *)
				   g = [Prim2(OpEq, Cst(CstI 9),Cst(CstI 7))]

				andalso (* internal var will not be const *)
				   not(SE.defined (#cenv sys) (S.symbol "var1"))

				andalso (* external var is now a const *)
					SE.defined (#cenv sys) (S.symbol "var3")

				andalso (* const remains as it was *)
					SE.defined (#cenv sys) 
						(S.symbol "myconst")

				andalso (* var1 and var3 are undeclared *)
					not(SE.defined (#vartypes sys)
						(S.symbol "var1"))
				andalso not(SE.defined (#vartypes sys)
						(S.symbol "var3"))
				andalso not(SE.defined (#venv sys)
						(S.symbol "var1"))
				andalso not(SE.defined (#venv sys)
						(S.symbol "var3"))
						
				andalso (* var2 is declared still *)
					SE.defined (#vartypes sys)
						(S.symbol "var2")
				andalso SE.defined (#venv sys)
						(S.symbol "var2")

				end)),
		
		FAILURE "test/run/spec03.vsp" `
		
			/* 14 type mismatch */
			restriction SPEC03 { const VS_FLOAT var1 = 9.0; };
			SPEC03 spec03;
		
		`,

		FAILURE "test/run/spec03.vsp" `

			/* 15 undeclared variable */
			restriction SPEC03 { const VS_FLOAT nonvar = 1; };
			SPEC03 spec03;

		`,

		SUCCESS "test/run/actions01.vsp" `

			/* check if purity is propagated */
			restriction ACTIONS01 { pure VS_VOID action(); };
			ACTIONS01 actions01;
		
		` (all2 (fn (_,sys) => 
				#3 (SE.lookup (#otypes sys) 
				(S.symbol "action")))),
			
		SUCCESS "test/run/actions01.vsp" `

			/* check if purity is not propagated */
			restriction ACTIONS01 { };
			ACTIONS01 actions01;
		
		` (all2 (fn (_,sys) => 
				not (#3 (SE.lookup (#otypes sys) 
					(S.symbol "action")))))
			

	]; (* end of testcases *)


	fun run (name,rl) =
		let val m = #1 (valOf (Main.parse_VS name))
		    val rl = valOf (tRLCommon.flatten rl)
		in 
			(m,SOME (Specialize.apply rl name m)) 
			handle (Err.ScopeExn _) => (m,NONE)
		end 
		

end


structure TF = TestFn(tSpecialize);

TF.go();
