(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tparserlCases.sml,v
   Comment: Cases for RL parser tests.    
   Release: scope-0-16_final

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   tparserlCases.sml,v 1.2 2003/07/08 15:45:06 wasowski Exp
*)

type testcase = unit frag list * token list option * rl_ast option

val testcases :testcase list = 
	[(
		`ala ma kota`,
		SOME [T_NAME "ala", T_NAME "ma", T_NAME "kota"],
		NONE
	 ),
	 (
		`
		 /* simple test for impossibleE */ 
		 restriction n { 
			impossible  e(); 
			impossible  e2(),e1(int); 
		 };
		`, 
		SOME [T_RESTRICTION, T_NAME "n", T_LBR, 
				T_IMPOSSIBLE, T_NAME "e",
				T_PARENTHESIZED, T_SEMICOLON,
				T_IMPOSSIBLE, T_NAME "e2",
				T_PARENTHESIZED, T_COMMA, 
				T_NAME "e1", T_PARENTHESIZED,
				T_SEMICOLON,
		T_RBR, T_SEMICOLON],
		SOME [Inter("n", [], [NeverE "e", NeverE "e2", NeverE "e1"])]
	),
	 (
		` 
		  /* constrain an integer var */ 
		  restriction n { 
		  
			const VS_INT name = 2; 
		  };
		`, 
		SOME [T_RESTRICTION, T_NAME "n", T_LBR, T_CONST, T_NAME "VS_INT", 
			T_NAME "name", T_EQUALS, T_CSTI 2, T_SEMICOLON, 
			T_RBR, T_SEMICOLON],
		SOME [Inter("n", [], [ConstrV(IntT, "name", ValI 2)])]
	),
	(
		` 
		  /* constrain a real variable */ 
		  restriction r { 
			
			const VS_FLOAT name = 2.00; 
		  };
		`, 
		SOME [T_RESTRICTION, T_NAME "r", T_LBR, T_CONST, 
			T_NAME "VS_FLOAT", T_NAME "name", 
			T_EQUALS, T_CSTR 2.0, T_SEMICOLON,T_RBR,T_SEMICOLON],
		SOME [Inter("r", [], [ConstrV(FloatT, "name", ValR 2.0)])]
	),
	(	` 
		  /* test multiple restriction (commasep test) */ 
		  
		  restriction A restricts B,C,D { };
		  
		`,
		SOME [T_RESTRICTION, T_NAME "A", T_RESTRICTS, T_NAME "B",
			T_COMMA, T_NAME "C", T_COMMA, T_NAME "D", T_LBR, 
			T_RBR, T_SEMICOLON],
		  SOME [Inter("A", ["B", "C", "D"], [])]

	),(
		`
			/* komentarz */
		`, 
		SOME [], SOME []

	),(
		`		
			/* consecutive comments */
			/* this is the second one  */
			/* this is a good test indeed ! */
		`,
		SOME [], SOME []
	),(
		`
		 restriction restriction_name restricts name_1, name_n { 

			impossible eventname1(), eventname2(VS_INT); 
			const VS_INT name = 2, VS_INT name3 = 1; 
			dead VS_INT name, VS_INT name2; 

			const VS_INT f(int)=4, VS_INT g(int)=5; 
			pure VS_INT f(int); 
		 };
		`,
		SOME [T_RESTRICTION, T_NAME "restriction_name", 
			T_RESTRICTS, T_NAME "name_1", T_COMMA, T_NAME "name_n", T_LBR,
			T_IMPOSSIBLE, T_NAME "eventname1",
			T_PARENTHESIZED, T_COMMA, T_NAME "eventname2", T_PARENTHESIZED,
			T_SEMICOLON,
			T_CONST, T_NAME "VS_INT", T_NAME "name", 
				T_EQUALS, T_CSTI 2, T_COMMA, 
				T_NAME "VS_INT", T_NAME "name3", 
				T_EQUALS, T_CSTI 1,T_SEMICOLON,
			T_DEAD, T_NAME "VS_INT", T_NAME "name", T_COMMA, 
				T_NAME "VS_INT", T_NAME "name2",T_SEMICOLON,
			T_CONST, T_NAME "VS_INT", T_NAME "f", 
				T_PARENTHESIZED, T_EQUALS, T_CSTI 4, T_COMMA,
				T_NAME "VS_INT", T_NAME "g", T_PARENTHESIZED, 
				T_EQUALS, T_CSTI 5, T_SEMICOLON,
			T_PURE, T_NAME "VS_INT",T_NAME "f", T_PARENTHESIZED, 
				T_SEMICOLON,
		T_RBR,T_SEMICOLON],
		SOME [Inter("restriction_name", ["name_1", "name_n"], [
			NeverE "eventname1", 
			NeverE "eventname2",
			ConstrV(IntT, "name", ValI 2), 
			ConstrV(IntT, "name3", ValI 1),
			Dead(IntT, "name"), 
			Dead(IntT, "name2"),
			ConstrF(IntT, "f", ValI 4), 
			ConstrF(IntT, "g", ValI 5),
			Pure(IntT, "f")])]
	), (
		`
			restriction_name_1 systemname_1; 
			restriction_name_2 systemname_2;
		`,
		SOME [T_NAME "restriction_name_1", T_NAME "systemname_1", 
		   T_SEMICOLON, 
		   T_NAME "restriction_name_2", T_NAME "systemname_2",
		   T_SEMICOLON],
		SOME [IBinding("restriction_name_1", "systemname_1"),
		       IBinding("restriction_name_2", "systemname_2")]
	),(
		`
			restriction A {} system;
		`,
		SOME [T_RESTRICTION, T_NAME "A", T_LBR, T_RBR, T_NAME "system",
		      T_SEMICOLON],
		NONE
	)
]; (* testcases *)

val names = List.tabulate (length testcases, 
		fn x => ("snippet " ^ Int.toString x));
